﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.IO;

namespace www_teach
{
    public partial class Form1 : Form
    {
        www myWebserver;

        public Form1()
        {
            InitializeComponent();
            this.myWebserver = new www(rawHttpTextBox);
        }

        private void wwwStartBtn_Click(object sender, EventArgs e)
        {
            try
            {
                if (!string.IsNullOrEmpty(wwwdirTextBox.Text))
                {
                    this.myWebserver.changeRoot(wwwdirTextBox.Text);
                    this.myWebserver.start();
                    wwwStartBtn.Enabled = false;
                    wwwStopBtn.Enabled = true;
                }
            }
            catch (Exception error)
            {
                MessageBox.Show(error.Message);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.myWebserver != null)
                myWebserver.stop();
        }

        private void backBtn_Click(object sender, EventArgs e)
        {
            webBrowser1.GoBack();
        }

        private void fowardBtn_Click(object sender, EventArgs e)
        {
            webBrowser1.GoForward();
        }

        private void refreshBtn_Click(object sender, EventArgs e)
        {
            webBrowser1.Refresh();
        }

        private void goBtn_Click(object sender, EventArgs e)
        {
            webBrowser1.Navigate(uriTextBox.Text);
        }

        private void wwwStopBtn_Click(object sender, EventArgs e)
        {
            this.myWebserver.stop();
            wwwStartBtn.Enabled = true;
            wwwStopBtn.Enabled = false;
        }

        private void browseBtn_Click(object sender, EventArgs e)
        {
            try
            {
                folderBrowserDialog1.ShowDialog();
                if (!string.IsNullOrEmpty(folderBrowserDialog1.SelectedPath))
                {
                    wwwdirTextBox.Text = folderBrowserDialog1.SelectedPath;
                    this.myWebserver.changeRoot(wwwdirTextBox.Text);
                }
            }
            catch (Exception error)
            {
                MessageBox.Show(error.Message);
            }
        }

        private void uriTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            // Allow the user to press enter instead of clicking the Go button.
            if (e.KeyChar == (char)13)
            {
                goBtn_Click(null, null);
            }
        }
    }

    public class www
    {
        private Thread serverThread;
        private bool closing;
        private TextBox output;
        private IPEndPoint ipEnd;
        private Socket sock;
        private string wwwroot;

        public www(TextBox output)
        {
            this.output = output;
            closing = false;
        }

        public void start()
        {
            if (string.IsNullOrEmpty(wwwroot))
            {
                throw (new Exception("Webserver will not start without wwwroot defined."));
            }
            else
            {
                serverThread = new Thread(webserver);
                serverThread.Start();
            }
        }

        public void changeRoot(string wwwroot)
        {
            this.wwwroot = wwwroot;
        }

        private void webserver()
        {
            // http://socketprogramming.blogspot.com/2007/09/socket-for-beginner.html
            ipEnd = new IPEndPoint(IPAddress.Any, 80);
            sock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP);
            sock.Bind(ipEnd);
            sock.Listen(100);

            while (!closing)
            {
                Socket clientSock = sock.Accept();
                byte[] clientData = new byte[1024];
                int receivedBytesLen = clientSock.Receive(clientData);
                string clientDataInString = Encoding.ASCII.GetString(clientData, 0, receivedBytesLen);

                if (output.InvokeRequired)
                {
                    this.output.Invoke(new SetTextCallback(this.SetText),
                         new object[] { clientDataInString });
                }
                else
                {
                    SetText(clientDataInString);
                }

                // This is hackish so let me explain.
                // The entire GET request will look like this:
                    // GET / HTTP/1.1
                    // Accept: */*
                    // Accept-Language: en-us
                    // Accept-Encoding: gzip, deflate
                    // User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 1.1.4322; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30618)
                    // Host: localhost
                    // Connection: Keep-Alive
                // First, we grab the first line and put that into $getstring.
                // Next, split that apart using ' ' as the token.
                // Now we can assume that requests[1] will be the actual document requested.
                // The only special case I can think of are:
                    // HTML special characters (i.e. Space%20Here.bmp)
                    // '/' will be used by most browsers looking at the root of a folder.
                // We won't worry about the unusual special characters and server directives
                // and will instead only look for the two special cases lsited above.
                string getstring = clientDataInString.Substring(0, clientDataInString.IndexOf('\n'));
                string[] requests = getstring.Split(' ');
                string outputDocument = requests[1];
                outputDocument.Replace("%20", " ");
                if (outputDocument.Equals("/"))
                    outputDocument = "/index.html";

                try
                {
                    if (File.Exists(string.Concat(wwwroot, outputDocument)))
                    {
                        FileStream fs = new FileStream(string.Concat(wwwroot, outputDocument), FileMode.Open);
                        byte[] b = new byte[1024];
                        while (fs.Read(b, 0, b.Length) > 0)
                        {
                            clientSock.Send(b);
                        }
                        fs.Close();
                    }
                    else
                    {
                        byte[] error = Encoding.ASCII.GetBytes("(404) File Not Found");
                        clientSock.Send(error);
                    }

                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message);
                }

                clientSock.Close();
            }
        }

        private delegate void SetTextCallback(string text);

        private void SetText(string text)
        {
            this.output.Text += text;
        }

        public void stop()
        {
            closing = true;
            if (sock != null)
                sock.Close();
            if (serverThread != null && serverThread.IsAlive)
            {
                serverThread.Abort();
                serverThread.Join();
                serverThread = null;
            }
        }
    }
}
